package com.yuandian.dataflow.statemachine_old.operate;

import java.io.Serializable;

import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.yuandian.dataflow.statemachine.state.WorkerState;
import com.yuandian.dataflow.statemachine_old.StateFactory;
import com.yuandian.dataflow.statemachine_old.closure.GenericClosure;
import com.yuandian.dataflow.statemachine_old.rpc.OperateProcessor;
import com.yuandian.dataflow.statemachine_old.rpc.RaftResponse;
import com.yuandian.dataflow.utils.PacketsManager;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;

/**
 * 操作
 * 
 * @author eson
 */
@Slf4j
@Data
public class OperateOld implements Serializable {

    private static int DEFAULT_ASYNC_TIMEOUT = 5000;
    public static final int MAX_TASKS = 1000;
    public static PacketsManager packetsManager = new PacketsManager();

    public static enum OperateType {
        /**
         * 同步WorkerState状态. 
         */
        PUT_WORKERSTATE, 

        /**
         * 分配packets
         */
        ALLOCATE_PACKETS, 

 
        /**
         *  暂无想法
         */
        REMOVE;
    }

    private OperateType type;
    private Object value;

 

    public OperateOld(OperateType t, Object v) {
        this.type = t;
        this.value = v;
    }

    @java.lang.SuppressWarnings("unchecked")
    public <T> T getValue() {
        return (T) this.value;
    };

    public <T> void setValue(T value) {
        this.value = value;
        return;
    };

    /**
     * 调用操作设置
     * @param op 传入的操作类
     * @param closure 回调函数. Operate为返回值
     */
    @java.lang.SuppressWarnings("unchecked")
    public static void CallOperate(OperateOld op, GenericClosure closure) {
        // log.debug("CallOperate Value {}", op.<WorkerState>getValue());
        // 如果是leader 就直接提交
        if (StateFactory.isLeader()) { 
            StateFactory.applyOperate(op, closure);
            return;
        }

        // 非leader 转发请求 统一有leader处理
        var request = new OperateProcessor.OperateRequest();
        request.setOperate(op);
        try {
            StateFactory.rpcClientInvokeAsync(StateFactory.getLeaderId().getEndpoint(), request, new InvokeCallback() {
                        @Override
                        public void complete(Object result, Throwable err) {
                            // log.debug("Object result {}", result);
                            var resp = (RaftResponse) result;
                            closure.setResponse(resp);
                            closure.success(resp.getValue());
                            closure.run(Status.OK());
                        }

                    }, DEFAULT_ASYNC_TIMEOUT);
        } catch (InterruptedException | RemotingException e) {
            closure.failure(e.getMessage(), null);
            closure.run(new Status(100000, "invokeAsync fail"));
            log.info("{}", e.toString());
        }

    }


  
}
